/***********************************************************************
 *
 * PROJECT: PalmXQ
 * FILE:    main.c
 * AUTHOR:  Cheok Yan Cheng
 *
 * DESCRIPTION: Main entry point for PalmXQ.
 *
 * LAST REVISION:   24/02/2002
 *
 ***********************************************************************/

#include <PalmOS.h>
#include "palmxqrsc.h"
#include "data.h"
#include "protos.h"
#include "defs.h"

static Err StartApplication(void);
static void StopApplication(void);
static Boolean ApplicationHandleEvent(EventPtr event);
static void EventLoop(void);


static Boolean SaveGame(void)
{
    Boolean handled = false;
    FileHand fileHandle;
    Err err;

    fileHandle = FileOpen(0, saveGameName, FileType, CreatorId, fileModeReadWrite, NULL);

    if(fileHandle != NULL && err == errNone)
    {
        if(FileError(fileHandle) == 0)
        {
            FileWrite(fileHandle, Piece, sizeof(Piece), 1, NULL);
            FileWrite(fileHandle, Color, sizeof(Color), 1, NULL);
            FileWrite(fileHandle, &Side, sizeof(Side), 1, NULL);
            FileWrite(fileHandle, &XSide, sizeof(XSide), 1, NULL);
            FileWrite(fileHandle, &Ply, sizeof(Ply), 1, NULL);
            FileWrite(fileHandle, &HPly, sizeof(HPly), 1, NULL);
            FileWrite(fileHandle, &Fifty, sizeof(Fifty), 1, NULL);
            FileWrite(fileHandle, HistDat, sizeof(HistT), HPly, NULL);
            FileWrite(fileHandle, &ComputerSide, sizeof(ComputerSide), 1, NULL);
            FileWrite(fileHandle, &ComputerDepth, sizeof(ComputerDepth), 1, NULL);
            FileWrite(fileHandle, &MaxTime, sizeof(MaxTime), 1, NULL);
            FileClose(fileHandle);
//            FrmAlert(SaveAlert);
            handled = true;
        }
    }

    return handled;
}



/***********************************************************************
 *
 * FUNCTION:    StartApplication
 *
 * DESCRIPTION: Starts this application.
 *
 * PARAMETERS:  NONE
 *
 * RETURNED:    The error code. Error code is 0 if successful.
 *
 * COMMENT:     NONE
 *
 ***********************************************************************/

static Err StartApplication(void)
{
    EventType event;
    UInt32    depth;

    if(!CheckOSVersion())
    {
        return 0;
    }
    
    depth = 16;

    WinScreenMode( winScreenModeSet, NULL, NULL, &depth, NULL );
    
    InitGlobals();

    FrmGotoForm(MainForm);

    MemSet( &event, sizeof( EventType ), 0 );
    event.eType = LOAD_EVENT;

    EvtAddEventToQueue( &event );

    return 0;
}

/***********************************************************************
 *
 * FUNCTION:    StopApplication
 *
 * DESCRIPTION: Stops this application.
 *
 * PARAMETERS:  NONE
 *
 * RETURNED:    NONE
 *
 * COMMENT:     NONE
 *
 ***********************************************************************/
static void StopApplication(void)
{
    SaveGame();
}

/***********************************************************************
 *
 * FUNCTION:    ApplicationHandleEvent
 *
 * DESCRIPTION: Event handling function for this application.
 *
 * PARAMETERS:  Event pointer to an event structure.
 *					
 * RETURNED:    True if event was handled. Else false.
 *
 * COMMENT:     NONE
 *
 ***********************************************************************/
static Boolean ApplicationHandleEvent(EventPtr event)
{
    FormType  *form;
    UInt16    formID;
    Boolean   handled = false;
    

    if (event->eType == frmLoadEvent)
    {
        formID = event->data.frmLoad.formID;
        form = FrmInitForm(formID);
        FrmSetActiveForm(form);
        
        switch (formID)
        {
            case MainForm:
                FrmSetEventHandler(form, MainHandleEvent);
                break;

            case SetupForm:
                FrmSetEventHandler(form, SetupHandleEvent);
                break;

            case LevelForm:
                FrmSetEventHandler(form, LevelHandleEvent);

            default:
                break;
        }
        handled = true;
    }

    return handled;
}

/***********************************************************************
 *
 * FUNCTION:    EventLoop
 *
 * DESCRIPTION: Performs infinity event looping until appStopEvent was
 *              encountered.
 *
 * PARAMETERS:  NONE
 *					
 * RETURNED:    NONE
 *
 * COMMENT:     NONE
 *
 ***********************************************************************/
static void EventLoop(void)
{
    EventType  event;
    UInt16     error;
    
    do
    {
        EvtGetEvent(&event, evtWaitForever);
        if (! SysHandleEvent(&event))
        {
            if (! MenuHandleEvent(0, &event, &error))
            {
                if (! ApplicationHandleEvent(&event))
                {
                    FrmDispatchEvent(&event);
                }
            }
        }
    } while (event.eType != appStopEvent);
}

UInt32 Go( void )
{
    Err err;

    err = StartApplication();
    if (errNone == err)
    {
        EventLoop();
        StopApplication();
    }

    return err;
}



/***********************************************************************
 *
 * FUNCTION:	PilotMain
 *
 * DESCRIPTION: Main entrypoint for the application.
 *
 * PARAMETERS:  cmd         Application Launchcode.
 *				cmbBPB		Pointer to a structure that is 
 *							associated with the launch code. 		
 *				launchFlags	value providing extra information about 
 *                          the launch.
 *					
 * RETURNED:    Error code. Error code is 0 if successful.
 *
 ***********************************************************************/
 
UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP,
                 UInt16 launchFlags)
{
    Err  err;

    switch (launchCode)
    {
        case sysAppLaunchCmdNormalLaunch:
            Go();
            break;

        default:
            break;
    }

    return err;
}
