/***********************************************************************
 *
 * PROJECT: PalmXQ
 * FILE:    search.c
 * AUTHOR:  Cheok Yan Cheng
 *
 * DESCRIPTION: Main Search Engine.
 *
 * LAST REVISION:   24/02/2002
 *
 ***********************************************************************/

#include <PalmOS.h>
#include <PceNativeCall.h>
#include <ErrorMgr.h>
#include <ErrorBase.h>
#include "palmxqrsc.h"
#include "data.h"
#include "protos.h"
#include "defs.h"
#include "stubs.h"

static Int16 AlphaBeta(ARMGLOBALS_PROTO Int16 alpha, Int16 beta, Int16 depth);
static Int16 QuiesceSearch(ARMGLOBALS_PROTO Int16 alpha, Int16 beta);
static void SortPV(ARMGLOBALS_PROTO_VOID);
static void Sort(ARMGLOBALS_PROTO Int16 from);
static Boolean CheckUp(ARMGLOBALS_PROTO_VOID);

void Think(ARMGLOBALS_PROTO_VOID)
{
    Int16 i;
    Int16 score;
    Err err;

#ifndef ARM
    MemHandle armH;
    UInt32    processor;
    Boolean   didARM;

    didARM = false;

    if ( errNone == FtrGet( sysFileCSystem, sysFtrNumProcessorID, &processor ) &&
                      sysFtrNumProcessorIsARM( processor ) ) {
        MemHandle armH;

        armH = DmGetResource( 'armc', armThink );

        if ( armH != NULL ) {
            void* f = MemHandleLock( armH );
            GameGlobals* copy;

            copy = MemPtrNew( ( sizeof( GameGlobals ) + 3 ) / 4 * 4 );
            if ( copy != NULL ) {
                MemMove( copy, &Globals, sizeof( GameGlobals ) );

                PceNativeCall( f, copy );

                MemMove( &Globals, copy, sizeof( GameGlobals ) );
                MemPtrFree( copy );
                didARM = true;
            }
            MemHandleUnlock( armH );
            DmReleaseResource( armH );
        }
    }

    if ( didARM )
        return;
#endif

    /* Root of the search tree */
    Ply = 0;
    Node = 0;

    /* Reset the content of priciple variation */
    MemSet(PV, sizeof(PV), 0);

    /* Reset the content of history heuristic */
    MemSet(History, sizeof(History), 0);

    /* We want to follow principle variation if possible */
    FollowPV = true;

    Time = TimGetTicks();
    StopTime = Time + MaxTime * SysTicksPerSecond();

#ifndef ARM
    ErrTry
#else
    Terminate = false;
#endif
    {
        for(i = 0; i <= ComputerDepth; i++)
        {
            score = AlphaBeta(ARMGLOBALS -10000, 10000, i);
            if(score > 9000 || score < -9000)
            {
                break;
            }
#ifdef ARM
            if ( Terminate ) {
                break;
            }
#endif
        }
    }
#ifndef ARM
    ErrCatch(err)
    {
    }
    ErrEndCatch
#endif

    while(Ply)
    {
        UnMakeMove(ARMGLOBALS_VOID);
    }

    Time = ( TimGetTicks() - Time + SysTicksPerSecond() / 2 ) / SysTicksPerSecond();
}

static Int16 AlphaBeta(ARMGLOBALS_PROTO Int16 alpha, Int16 beta, Int16 depth)
{
    Int16 i;
    Int16 j;
    Int16 score;
    Boolean found;

    if(depth <= 0)
    {
        return QuiesceSearch(ARMGLOBALS alpha, beta);
    }

    Node++;

    if((Node & 2047) == 0)
    {
#ifdef ARM
        if ( CheckUp(ARMGLOBALS_VOID) ) {
            Terminate = true;
            return 0;
        }
#else
        CheckUp(ARMGLOBALS_VOID);
#endif
    }

    PVLength[Ply] = Ply;

    if(Ply && Repeat(ARMGLOBALS_VOID))
    {
        // We assume this line is draw
        return 0;
    }

    if(Ply >= MAX_PLY - 1 || HPly >= HIST_STACK - 1)
    {
        return Eval(ARMGLOBALS_VOID);
    }

    if(IsInCheck(ARMGLOBALS Side))
    {
        depth++;
    }
    else
    {
        // Try null move forward pruning
        if((Ply & 1) == 0)
        {
            FirstMove[Ply+1]=FirstMove[Ply];
            MakeNullMove(ARMGLOBALS_VOID);
            score = -AlphaBeta(ARMGLOBALS -beta, -beta + 1, depth - 1 - R);
#ifdef ARM
            if ( Terminate )
                return 0;
#endif
            UnMakeNullMove(ARMGLOBALS_VOID);

            if(score >= beta)
            {
                return beta;
            }
        }
    }

    Gen(ARMGLOBALS_VOID);

    // Are we still in PV line
    if(FollowPV)
    {
        SortPV(ARMGLOBALS_VOID);
    }

    found = false;
    for(i = FirstMove[Ply]; i < FirstMove[Ply +1]; i++)
    {
        Sort(ARMGLOBALS i);

        if(!MakeMove(ARMGLOBALS GenDat[i].m))
        {
            continue;
        }

        found = true;
        score = -AlphaBeta(ARMGLOBALS -beta, -alpha, depth - 1);
#ifdef ARM
        if ( Terminate )
            return 0;
#endif
        UnMakeMove(ARMGLOBALS_VOID);

        if(score > alpha)
        {
            History[GenDat[i].m.b.from][GenDat[i].m.b.to] += depth;

            if(score >= beta)
            {
                return beta;
            }

            alpha = score;

            PV[Ply][Ply] = GenDat[i].m;

            for(j = Ply + 1; j < PVLength[Ply + 1]; j++)
            {
                PV[Ply][j] = PV[Ply + 1][j];
            }

            PVLength[Ply] = PVLength[Ply + 1];
        }
    }

    if(found == false)
    {
        return -10000 + Ply;
    }

    if(Fifty >= 100)
    {
        return 0;
    }

    return alpha;
}

static Int16 QuiesceSearch(ARMGLOBALS_PROTO Int16 alpha, Int16 beta)
{
    Int16 score;
    Int16 i;
    Int16 j;

    Node++;

    if((Node & 2047) == 0)
    {
#ifdef ARM
        if ( CheckUp(ARMGLOBALS_VOID) ) {
            Terminate = true;
            return 0;
        }
#else
        CheckUp(ARMGLOBALS_VOID);
#endif
    }

    PVLength[Ply] = Ply;

    if(Ply >= MAX_PLY - 1)
    {
        return Eval(ARMGLOBALS_VOID);
    }

    if(HPly >= HIST_STACK - 1)
    {
        return Eval(ARMGLOBALS_VOID);
    }

    // Check with the evaluation function
    score = Eval(ARMGLOBALS_VOID);

    if (score >= beta)
    {
		return beta;
    }

    if (score > alpha)
    {
        alpha = score;
    }

    GenCap(ARMGLOBALS_VOID);

    if (FollowPV)
    {
        SortPV(ARMGLOBALS_VOID);
    }

    // Loop through the moves
    for (i = FirstMove[Ply]; i < FirstMove[Ply + 1]; i++)
    {
        Sort(ARMGLOBALS i);
        if (!MakeMove(ARMGLOBALS GenDat[i].m))
        {
        	continue;
        }
        score = -QuiesceSearch(ARMGLOBALS -beta, -alpha);
#ifdef ARM
        if ( Terminate ) {
            return 0;
        }
#endif
        UnMakeMove(ARMGLOBALS_VOID);

        if (score > alpha)
        {
            if (score >= beta)
            {
				return beta;
            }
            alpha = score;

            // Update the PV
            PV[Ply][Ply] = GenDat[i].m;
            for (j = Ply + 1; j < PVLength[Ply + 1]; j++)
            {
                PV[Ply][j] = PV[Ply + 1][j];
            }
            PVLength[Ply] = PVLength[Ply + 1];
		}
	}
	return alpha;
}

static void SortPV(ARMGLOBALS_PROTO_VOID)
{
    Int16 i;

    FollowPV = false;
    for(i = FirstMove[Ply]; i < FirstMove[Ply + 1]; i++)
    {
        if (GenDat[i].m.u == PV[0][Ply].u)
        {
            FollowPV = true;
            GenDat[i].score += 1000;
            break;
		}
    }
}

void Sort(ARMGLOBALS_PROTO Int16 from)
{
    Int16 i;
    Int16 bs;  /* best score */
    Int16 bi;  /* best i */
    GenT gen;

	bs = -1;
	bi = from;
    for (i = from; i < FirstMove[Ply + 1]; i++)
    {
        if (GenDat[i].score > bs)
        {
            bs = GenDat[i].score;
			bi = i;
		}
    }

    gen = GenDat[from];
    GenDat[from] = GenDat[bi];
    GenDat[bi] = gen;
}

static Boolean CheckUp(ARMGLOBALS_PROTO_VOID)
{
    if(TimGetTicks() >= StopTime)
    {
#ifdef ARM
        return true;
#else
        ErrThrow(0);
#endif
    }
    return false;
}
